# -*- coding: UTF-8 -*-
# @Time : 2021/11/27 下午6:15 
# @Author : 刘洪波


class Producer(object):
    def __init__(self, client, topic, schema):
        self.producer = client.create_producer(topic, schema=schema)

    def send(self, msg, _async=True, callback=None, logger=None):
        """
        发送消息
        :param msg: 需要发送的消息
        :param _async: 是否异步发送消息， True异步发送， Flase 同步发送
        :param callback: 异步发送时的回调函数
        :param logger: 日志收集器
        :return:
        """
        try:
            if _async:
                self.producer.send_async(msg, callback=callback)
            else:
                self.producer.send(msg)
        except Exception as e:
            if logger:
                logger.error('生产异常，报错信息如下：')
                logger.error(e)
            else:
                print('生产异常，报错信息如下：')
                print(e)
            raise e



# -*- coding: UTF-8 -*-
# @Time : 2021/11/28 上午12:03 
# @Author : 刘洪波
import pulsar


class Client(object):
    def __init__(self, url: str):
        self.client = pulsar.Client(url)

    def create_consumer(self, topic: str, consumer_name: str, schema=pulsar.schema.StringSchema()):
        """
        创建 消费者
        :param topic: str
        :param consumer_name: 消费者名字
        :param schema:
        :return:
        """
        from pulsar_mq.Consumers import Consumer
        return Consumer(self.client, topic, consumer_name, schema)

    def create_producer(self, topic: str, schema=pulsar.schema.StringSchema()):
        """
        创建生产者
        :param topic: topic
        :param schema:
        :return:
        """
        from pulsar_mq.Producers import Producer
        return Producer(self.client, topic, schema)

    def service(self, producer_topic: str, consumer_topic: str, consumer_name: str,
                schema=pulsar.schema.StringSchema()):
        """
        pulsar 消费数据后 并且发送数据的 服务
        1. 订阅pulsar
        2. 处理消费的数据
        3. 发送得到的结果
        :param producer_topic:
        :param consumer_topic:
        :param consumer_name:
        :param schema:
        :return:
        """
        from pulsar_mq.Services import Service
        return Service(self.client, producer_topic, consumer_topic, consumer_name, schema)

    def close(self):
        """
        关闭 client
        :return:
        """
        self.client.close()



# -*- coding: UTF-8 -*-
# @Time : 2021/12/3 下午7:05 
# @Author : 刘洪波
import pulsar_mq
import rabbitmq
from concurrent.futures import ThreadPoolExecutor

"""rabbitmq 和 pulsar互相通信"""


class Interconnect(object):
    def __init__(self, rabbitmq_host, rabbitmq_port, rabbitmq_username, rabbitmq_password, pulsar_url):
        """
        rabbitmq 和 pulsar 连接
        :param rabbitmq_host: rabbitmq 的 host
        :param rabbitmq_port: rabbitmq 的 port
        :param rabbitmq_username: rabbitmq 的 username
        :param rabbitmq_password: rabbitmq 的 password
        :param pulsar_url: pulsar的url
        """
        self.rabbitmq_connect = rabbitmq.connect(rabbitmq_host, rabbitmq_port, rabbitmq_username, rabbitmq_password)
        self.client = pulsar_mq.client(pulsar_url)

    def rabbitmq_to_pulsar(self, task, pulsar_topic, rabbitmq_exchange, rabbitmq_routing_key, durable=False,
                           rabbitmq_thread_count=None, _async=True, send_callback=None, logger=None):
        """
        1. 订阅rabbitmq
        2. 处理消费的数据
        3. 将处理后的数据 发送到 pulsar
        :param task:  该函数是处理消费的数据
        :param pulsar_topic:
        :param rabbitmq_exchange:
        :param rabbitmq_routing_key:
        :param durable:
        :param rabbitmq_thread_count:
        :param _async:  pulsar是否异步发送
        :param send_callback: pulsar异步发送时的回调函数
        :param logger: 日志收集器
        :return:
        """
        try:
            producer = self.client.create_producer(pulsar_topic)

            def callback(msg):
                if logger:
                    result = task(msg, logger)
                else:
                    result = task(msg)
                if result:
                    producer.send(result, _async, send_callback, logger)

            self.rabbitmq_connect.receive(rabbitmq_exchange, rabbitmq_routing_key, callback,
                                          durable, rabbitmq_thread_count)
        except Exception as e:
            if logger:
                logger.error(e)
            else:
                print(e)

    def pulsar_to_rabbitmq(self, task, pulsar_topic, pulsar_consumer_name, rabbitmq_exchange, rabbitmq_routing_key,
                           durable=False, pulsar_thread_count=None, logger=None):
        """
        1. 订阅 pulsar
        2. 处理消费的数据
        3. 将处理后的数据发送到 rabbitmq
        :param task:
        :param pulsar_topic:
        :param pulsar_consumer_name:
        :param rabbitmq_exchange:
        :param rabbitmq_routing_key:
        :param pulsar_thread_count:
        :param durable:
        :param logger:
        :return:
        """
        try:
            consumer = self.client.create_consumer(pulsar_topic, pulsar_consumer_name)

            def callback(msg):
                if logger:
                    result = task(msg, logger)
                else:
                    result = task(msg)
                if result:
                    self.rabbitmq_connect.send(result, rabbitmq_exchange, rabbitmq_routing_key, durable)

            consumer.receive(callback, pulsar_thread_count, logger)
        except Exception as e:
            if logger:
                logger.error(e)
            else:
                print(e)

    def run(self, pulsar_producer_topic, pulsar_consumer_topic, pulsar_consumer_name,
            rabbitmq_send_exchange, rabbitmq_send_routing_key, rabbitmq_consumer_exchange,
            rabbitmq_consumer_routing_key, send_task=None, consumer_task=None, durable=False,
            consumer_durable=None, producer_durable=None,
            _async=True, send_callback=None, rabbitmq_thread_count=None, pulsar_thread_count=None,
            logger=None):
        """
        从 rabbitmq 订阅，将数据发送至 pulsar;
        并且从 pulsar 订阅，将数据发送至 rabbitmq
        :param send_task:
        :param consumer_task:
        :param pulsar_producer_topic:
        :param pulsar_consumer_topic:
        :param pulsar_consumer_name:
        :param rabbitmq_send_exchange:
        :param rabbitmq_send_routing_key:
        :param rabbitmq_consumer_exchange:
        :param rabbitmq_consumer_routing_key:
        :param durable:
        :param consumer_durable:
        :param producer_durable:
        :param _async:
        :param send_callback:
        :param rabbitmq_thread_count:
        :param pulsar_thread_count:
        :param logger:
        :return:
        """
        if logger:
            def send_task2(msg, loggers):
                loggers.info('rabbitmq的输入：')
                loggers.info(msg)
                return msg

            def consumer_task2(msg, loggers):
                loggers.info('pulsar的输出：')
                loggers.info(msg.data())
                return msg.data()
        else:
            def send_task2(msg): return msg

            def consumer_task2(msg): return msg.data()

        if send_task is None:
            send_task = send_task2
        if consumer_task is None:
            consumer_task = consumer_task2
        pool = ThreadPoolExecutor(max_workers=2)
        pool.submit(self.rabbitmq_to_pulsar, send_task, pulsar_producer_topic, rabbitmq_consumer_exchange,
                    rabbitmq_consumer_routing_key, durable=consumer_durable if consumer_durable else durable,
                    rabbitmq_thread_count=rabbitmq_thread_count,
                    _async=_async, send_callback=send_callback, logger=logger)
        pool.submit(self.pulsar_to_rabbitmq, consumer_task, pulsar_consumer_topic,
                    pulsar_consumer_name, rabbitmq_send_exchange, rabbitmq_send_routing_key,
                    durable=producer_durable if producer_durable else durable,
                    pulsar_thread_count=pulsar_thread_count, logger=logger)




# -*- coding: UTF-8 -*-
# @Time : 2021/12/3 下午6:36 
# @Author : 刘洪波


def rabbitmq_pulsar(host, port, username, password, url):
    from mq.Rabbitmq_pulsar import Interconnect
    return Interconnect(host, port, username, password, url)


# -*- coding: UTF-8 -*-
# @Time : 2021/12/1 下午4:04 
# @Author : 刘洪波

from concurrent.futures import ThreadPoolExecutor

"""
pulsar服务
1. 订阅pulsar
2. 处理消费的数据
3. 发送得到的结果
"""


class Service(object):
    def __init__(self, client, producer_topic: str, consumer_topic: str, consumer_name: str, schema):
        """
        创建 生产者 和 消费者
        :param client:
        :param producer_topic:
        :param consumer_topic:
        :param consumer_name:
        :param schema:
        """
        self.producer = client.create_producer(producer_topic, schema=schema)
        self.consumer = client.subscribe(consumer_topic, consumer_name, schema=schema)

    def run(self, task, thread_count=None, _async=True, callback=None, logger=None):
        """
        多线程处理
        :param task: 任务程序
        :param thread_count: 指定最大线程数
        :param _async: 是否异步发送消息， True异步发送， Flase 同步发送
        :param callback: 异步发送的回调函数
        :param logger: 日志收集器
        :return:
        """
        if thread_count:
            pool = ThreadPoolExecutor(max_workers=thread_count)
            while True:
                msg = self.consumer.receive()
                pool.submit(self.acknowledge, task, msg, _async, callback, logger)
        else:
            while True:
                msg = self.consumer.receive()
                self.acknowledge(task, msg, _async, callback, logger)

    def acknowledge(self, task, msg, _async, callback, logger):
        try:
            if logger:
                result = task(msg, logger)
            else:
                result = task(msg)
            if result:
                if _async:
                    self.producer.send_async(result, callback=callback)
                else:
                    self.producer.send(result)
            self.consumer.acknowledge(msg)
        except Exception as e:
            # 消息未被成功处理
            self.consumer.acknowledge(msg)
            if logger:
                logger.error('运行异常，报错信息如下：')
                logger.error(e)
            else:
                print('运行异常，报错信息如下：')
                print(e)
            raise e
